/* version  "00.01.00" major.minor.bugfix */
/* Sven Goethel, sgoethel@jausoft.com, LICENSE: LGPL */
/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA  02111-1307, USA.
*/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cddata.h"

const char * CDDBData::tocFile = "./cda-toc.txt" ;
const char * CDDBData::version = "00.01.00"; // major.minor.bugfix

bool CDDBData::get (const char * cddevice, bool verbose)
{
	if(cddevice==0) return false;

	doCDA(cddevice, verbose);

        FILE *filePtr;
        if ((filePtr = fopen(tocFile, "r")) == NULL) {
                return false;
        }

        int i;
        char trackname[180], trackname2[120], tracknum[4], tracklen[8];

        grabTitle (filePtr); // artist and album :-)

	if(verbose)
		printf("get %s: %s\n", artist, album);

        trackNumber=0;
        for (i=0; !feof(filePtr); i++){
                grabTrack (filePtr, tracknum, tracklen, trackname, trackname2);

		if(strcmp(tracknum, tracks[i-1].tracknum)==0)
			break;

		if(strcmp("??", trackname)==0)
		{
			sprintf(trackname,"track%s len%s", 	
				tracknum, tracklen);
		}
                if (!feof(filePtr) == 1){
                        tracks[i].action=false;
                        strcpy (tracks[i].tracknum,tracknum);
                        strcpy (tracks[i].tracklen,tracklen);
                        strcpy (tracks[i].trackname,trackname);
                        strcpy (tracks[i].trackname2,trackname2);
                        trackNumber++;
                }
		if(verbose)
		{
			printf("get %d: %s %s %s\n",
				i,
				tracknum, tracklen, trackname);
		}
        }

        fclose (filePtr);
	return true;
}

const char * CDDBData::getFileName(int trackIndex) const
{
	static char buffer[500];

	if(trackIndex>=trackNumber)
		return "exceeded trackIndex";

	sprintf(buffer, "%s-%s-%s.mp3", 
		artist, album, tracks[trackIndex].trackname );

	CDDBData::validateFileName(buffer);

	return buffer;
}

void CDDBData::validateFileName(char * filename)
{
        int pos = 0;
	int len = strlen(filename);
        while (pos<len)
	{
                if (filename[pos]=='\n') filename[pos]='_';
                if (filename[pos]==' ') filename[pos]='_';
                if (filename[pos]=='?') filename[pos]='_';
                if (filename[pos]=='<') filename[pos]='_';
                if (filename[pos]=='>') filename[pos]='_';
                if (filename[pos]==':') filename[pos]='_';
                if (filename[pos]=='*') filename[pos]='-';
                pos++;
        } 
} 

void CDDBData::grabTitle(FILE *filePtr)
{
	char buffer[200];
	int j, pos;
	char * l_artist = 0;
	char * l_album = 0;

        fgets(buffer, 200, filePtr);
        fgets(buffer, 200, filePtr);

	if(strstr(buffer, "unknown disc title")!=0)
	{
		strcpy(artist,"unknown / unknown");
	}
	l_artist = buffer ;  // artist is the first entry !

        for(pos=0; buffer[pos]!=0; pos++)
	{
		/* CUT the Album from the Artist */
                if (buffer[pos]=='/') 
		{
			buffer[pos]=' ';
			/* remove post blanks */	
			for(j=pos; j>=0 && buffer[j]==' '; j--)
				buffer[j]=0;

			pos++; // go to the album title
			while(buffer[pos]==' ') pos++;  // skip blanks
			
			l_album = buffer+pos;

			/* remove post blanks and cr */	
			for(j=strlen(l_album)-1; j>=0; j--)
			{
			    if(l_album[j]==' '|| l_album[j]=='\n' )
				l_album[j]=0;
			    else break; // ugly ugly
			}
		}
	}
	if(l_artist) strcpy(artist, l_artist);
	if(l_album) strcpy(album, l_album);
}

void CDDBData::grabTrack(FILE *filePtr, 
		 char *tracknum, char *tracklen, 
		 char *trackname, char *trackname2)
{
        int pos=0;
        int len=0;

        fscanf(filePtr, "%s", tracknum);
        fscanf(filePtr, "%s", tracklen);
        fscanf(filePtr, "%s", trackname);
        fgets(trackname2, 200, filePtr);
        sprintf (trackname,"%s%s",trackname, trackname2);

        len = strlen(trackname);

        do {
                if (trackname[pos]=='\n') trackname[pos]=0;
                pos++;
        } while (pos<len);
}

void CDDBData::doCDA(const char * cddevice, bool verbose) 
{
	char buffer[200];

	sprintf(buffer, "cda -dev %s on > /dev/null", cddevice);
	system(buffer);

	sprintf(buffer, "cda -dev %s stop > /dev/null", cddevice);
	system(buffer);

	sprintf(buffer, "cda -dev %s toc | grep -v '^Total Time' > %s", cddevice, tocFile);
	system(buffer);

	sprintf(buffer, "cda -dev %s off > /dev/null", cddevice);
	system(buffer);

	if(verbose)
	{
		sprintf(buffer, "cat %s", tocFile);
		system(buffer);
	}
}


