/*
 * @(#)mp3id3cddb	0.1 99/03/23
 *
 * main for mp3id3cddb
 *
 *	mp3id3cddb - Command-line MP3 CDDB (cda) MP3ID3 Writer
 *
 *   Copyright (C) 1999  Sven Goethel
 *   E-mail: sgoethel@jausoft.com
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*=============================================================================
  HEADERs
 =============================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

#include "mp3id3util.h"
#include "cddata.h"

/*=============================================================================
  GLOBALs
=============================================================================*/
extern int      errno;

static int verbose = 0;
static int simulate = 0;
static int file_rename = 0;
static int startTrack = 0;

static const char * cddevice = 0;

static const char * artist = 0;
static const char * album = 0;
static const char * year = 0;
static const char * comment = 0;
static int genre = Tag::genre_largest;

static const int MAX_FILES = 100;
static const int FILENAME_LEN = 250;
static char * files[MAX_FILES];
static int fileNumber=0;

/* JAU */
void usage(char *progname)
{
	(void) fprintf(stderr, "Usage: %s %s%s%s%s%s%s%s%s", progname,
		"-cddev device\n",
		"\t[-v] [-listgenres]\n",
		"\t[-t startTrack]\n",
		"\t[-setartist string] [-setalbum string] \n", 
		"\t[-setyear string] [-setcomment string] [-setgenre int] \n",
		"\t[-simulate]\n",
		"\t[-rename]\n",
		"-ifiles mp3file1 mp3file2 ...  \n"
               );
	(void) fprintf(stderr, "%s%s%s%s%s%s%s",
		"\t\t-cddev device the cdrom device\n",
		"\t\t-v \tverbose output\n",
		"\t\t-t startTrack set cd-starttrack index [1-..] assigned to the first file of the given files\n",
		"\t\t-simulate simulate - not modifying the mp3-file\n",
		"\t\t-rename renames files to: artist-album-title.mp3\n",
		"\t\t-listgenres list all genres in mp3 tag\n",
		"\t\t-ifiles mp3file the mp3 files MUST BE AT LAST !\n" );
}

/* JAU */
int parse_args(int argc, char **argv)
{
	int	i;

	for (i = 1; i < argc; i++) {
		if (strcmp(argv[i], "-listgenres") == 0) {
			mp3id3ListGenres(stdout);
		}
		if (strcmp(argv[i], "-rename") == 0) {
			file_rename=1;
		}
		if (strcmp(argv[i], "-simulate") == 0) {
			simulate=1;
		}
		if (strcmp(argv[i], "-v") == 0) {
			verbose=1;
		}
		if (strcmp(argv[i], "-t") == 0) {
			if (++i < argc) {
				startTrack = atoi(argv[i]);
				// [1..number] -> [0..number-1] :-|
				startTrack-- ; 
			}
			else {
				usage(argv[0]);
				return 0;
			}
		}
		if (strcmp(argv[i], "-cddev") == 0) {
			if (++i < argc) {
				cddevice = argv[i];
			}
			else {
				usage(argv[0]);
				return 0;
			}
		}
		if (strcmp(argv[i], "-setartist") == 0) {
			if (++i < argc) {
				artist = argv[i];
			}
			else {
				usage(argv[0]);
				return 0;
			}
		}
		if (strcmp(argv[i], "-setalbum") == 0) {
			if (++i < argc) {
				album = argv[i];
			}
			else {
				usage(argv[0]);
				return 0;
			}
		}
		if (strcmp(argv[i], "-setgenre") == 0) {
			if (++i < argc) {
				genre = atoi(argv[i]);
			}
			else {
				usage(argv[0]);
				return 0;
			}
		}
		if (strcmp(argv[i], "-setcomment") == 0) {
			if (++i < argc) {
				comment = argv[i];
			}
			else {
				usage(argv[0]);
				return 0;
			}
		}
		if (strcmp(argv[i], "-setyear") == 0) {
			if (++i < argc) {
				year = argv[i];
			}
			else {
				usage(argv[0]);
				return 0;
			}
		}
		if (strcmp(argv[i], "-ifiles") == 0) {
			while (++i < argc) {
				files[fileNumber] = (char *)malloc(FILENAME_LEN);	
				strcpy(files[fileNumber], argv[i]);
				fileNumber++;
			}
		}
	}
	if(verbose)
	{
		printf("verbose mode\n");
		printf("simulate = %d\n", (int)simulate);
		printf("rename = %d\n", (int)file_rename);
		printf("cdromdev = %s\n", cddevice);
		printf("artist = %s\n", artist);
		printf("album = %s\n", album);
		printf("year = %s\n", year);
		printf("genre = %d -> %s\n", genre, Tag::genres[genre]);
		printf("comment = %s\n", comment );
		printf("ifiles (%d):\n", fileNumber);
		for(i=0; i<fileNumber; i++)
			printf("file[%d] = %s\n", i, files[i] );
	}
	if(fileNumber==0 || cddevice==0 ) {
		usage(argv[0]);
		return 0;
	}

	return 1;
}

int main(int argc, char **argv)
{
  CDDBData cd ;
  int i;

  if( parse_args(argc, argv) == 0)
	exit(1);

  cd.get(cddevice, verbose);

  if( ! ( strcmp(cd.artist,"unknown")==0 && artist!=0) )
	artist=(const char *) &cd.artist;

  if( ! ( strcmp(cd.album,"unknown")==0 && album!=0) )
	album=(const char *) &cd.album;

  for(i=0; i<fileNumber && i+startTrack<cd.trackNumber; i++)
  {
	  FILE * fd2 = fopen(files[i], "r+");
	  if(!fd2) {
		fprintf(stderr, "ERROR, could not open file %s\n", files[i]);
		continue ; // next file ...
	  }

	  Tag * tag = 0;

	  bool isTag = mp3id3Get(fd2, &tag);

	  CDtrack & track = cd.tracks[i+startTrack];

	  if(!isTag)
		  printf("no tag is provided in file %s\n", files[i]);

	  if(verbose || simulate)
	  {
	  	printf("set MP3ID3 of file:%s\n", files[i]);
	  }

	  if(isTag && ( verbose || simulate ) )
	  {
		  mp3id3Show(tag);
	  }

	  if(!simulate) 
	  {
		  (void) mp3id3Set(fd2, tag, verbose, track.trackname, 
				   cd.artist, cd.album, 
				   year, comment, genre ) ;
          } else {
	  	printf("title :%s\n", track.trackname);
	  	printf("artist :%s\n", cd.artist);
	  	printf("album :%s\n", cd.album);
	  	printf("year :%s\n", year);
	  	printf("comment :%s\n", comment);
	  	printf("genre :%d -> %s\n", genre, Tag::genres[genre]);
	  }

	  fclose(fd2);
	  fd2=0;
	  delete tag;
	  tag = 0;
	  if(file_rename)
	  {
	  	char fname[400];
		strcpy(fname, cd.getFileName(i+startTrack));

	  	char buffer[400];
		sprintf(buffer, "mv \"%s\" \"%s\"", files[i], fname);

		if(verbose || simulate)
			printf(">%s\n", buffer);
		if(!simulate)
			system(buffer);
	  }
	  
  }
  exit(0);

}
