/*
 * @(#)mp3id3mod	0.1 99/03/23
 *
 * main for mp3id3mod
 *
 *	mp3id3mod - Command-line MP3 Header Viewer
 *
 *   Copyright (C) 1999  Sven Goethel
 *   E-mail: sgoethel@jausoft.com
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*=============================================================================
  HEADERs
 =============================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>


#include "mp3id3util.h"

/*=============================================================================
  GLOBALs
=============================================================================*/
extern int      errno;

static int verbose = 0;
static int showmp3t = 0;
static const char * fileName = 0;

static const char * title = 0;
static const char * artist = 0;
static const char * album = 0;
static const char * year = 0;
static const char * comment = 0;
static int genre = 0;

/* JAU */
void usage(char *progname)
{
	(void) fprintf(stderr, "Usage: %s %s%s%s%s", progname,
		"-if mp3file \n",
		"\t[-v] [-showmp3t] [-listgenres]\n",
		"\t[-settitle string] [-setartist string] [-setalbum string] \n", 
		"\t[-setyear string] [-setcomment string] [-setgenre int] \n"
               );
	(void) fprintf(stderr, "%s%s%s%s",
		"\t\t-if mp3file \tthe mp3 file\n",
		"\t\t-v \tverbose output\n",
		"\t\t-showmp3t \tshow mp3 tag\n",
		"\t\t-listgenres \tlist all genres in mp3 tag\n" );
}

/* JAU */
int parse_args(int argc, char **argv)
{
	int	i;

	for (i = 1; i < argc; i++) {
		if (strcmp(argv[i], "-showmp3t") == 0) {
			showmp3t=1;
		}
		if (strcmp(argv[i], "-listgenres") == 0) {
			mp3id3ListGenres(stdout);
		}
		if (strcmp(argv[i], "-v") == 0) {
			verbose=1;
		}
		if (strcmp(argv[i], "-if") == 0) {
			if (++i < argc) {
				fileName = argv[i];
			}
			else {
				usage(argv[0]);
				return 0;
			}
		}
		if (strcmp(argv[i], "-settitle") == 0) {
			if (++i < argc) {
				title = argv[i];
			}
			else {
				usage(argv[0]);
				return 0;
			}
		}
		if (strcmp(argv[i], "-setartist") == 0) {
			if (++i < argc) {
				artist = argv[i];
			}
			else {
				usage(argv[0]);
				return 0;
			}
		}
		if (strcmp(argv[i], "-setalbum") == 0) {
			if (++i < argc) {
				album = argv[i];
			}
			else {
				usage(argv[0]);
				return 0;
			}
		}
		if (strcmp(argv[i], "-setgenre") == 0) {
			if (++i < argc) {
				genre = atoi(argv[i]);
			}
			else {
				usage(argv[0]);
				return 0;
			}
		}
		if (strcmp(argv[i], "-setcomment") == 0) {
			if (++i < argc) {
				comment = argv[i];
			}
			else {
				usage(argv[0]);
				return 0;
			}
		}
		if (strcmp(argv[i], "-setyear") == 0) {
			if (++i < argc) {
				year = argv[i];
			}
			else {
				usage(argv[0]);
				return 0;
			}
		}
	}
	if(fileName==0)
	{
		usage(argv[0]);
		return 0;
	}

	if(verbose)
	{
		printf("verbose mode\n");
		printf("filename = %s\n", fileName);
		printf("title = %s\n", title);
		printf("artist = %s\n", artist);
		printf("album = %s\n", album);
		printf("year = %s\n", year);
		printf("genre = %d -> %s\n", genre, Tag::genres[genre]);
		printf("comment = %s\n", comment );
	}
	return 1;
}

int main(int argc, char **argv)
{
  FILE *  fd2=0;

  if( parse_args(argc, argv) == 0)
	exit(1);

  fd2 = fopen(fileName, "r+");
  if(!fd2) {
    fd2 = fopen(fileName, "r");
    fprintf(stderr, "Warning, file %s opened in read-only mode\n", fileName);
    if(!fd2) {
        fprintf(stderr, "ERROR, could not open file %s\n", fileName);
	exit(1);
    }
  }

  Tag * tag = 0;

  bool isTag = mp3id3Get(fd2, &tag);

  if(!isTag)
	  printf("no tag is provided in file %s\n", fileName);

  if(isTag && showmp3t)
  {
	  mp3id3Show(tag);
  }

  (void) mp3id3Set(fd2, tag, verbose, title, artist, album, 
	           year, comment, genre ) ;
  fclose(fd2);
  fd2=0;
  delete tag;
  tag = 0;
  exit(0);

}
