/*
 * @(#)mp3id3	0.1 99/03/23
 *
 * utilities for mp3id3mod
 *
 *	mp3id3mod - Command-line MP3 Header Viewer
 *
 *   Copyright (C) 1999  Sven Goethel
 *   E-mail: sgoethel@jausoft.com
 *

     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
     License as published by the Free Software Foundation; either
     version 2 of the License, or (at your option) any later version.
     
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Library General Public License for more details.
     
     You should have received a copy of the GNU Library General Public
     License along with this library; if not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA  02111-1307, USA.
     
 */

/*=============================================================================
  HEADERs
 =============================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>


#include "mp3id3util.h"

/*=============================================================================
  GLOBALs
=============================================================================*/
extern int      errno;

void mp3id3ListGenres(FILE *out)
{
	int i;
	for(i=0; i< Tag::genre_largest+1; i++)
	{
		fprintf(out, "%d: %s\n", i, Tag::genres[i]); 
	}
}

void  mp3id3Show(const Tag * inTag)
{
  if(inTag!=0)
  {
	  printf("title: '%s'\n", inTag->getTitle());
	  printf("album: '%s'\n", inTag->getAlbum());
	  printf("artist: '%s'\n", inTag->getArtist());
	  printf("year: '%s'\n", inTag->getYear());
	  printf("comment: '%s'\n", inTag->getComment());
	  printf("genre: %d -> '%s'\n", 
		inTag->getGenreNum(), inTag->getGenre());
  } 
}

bool mp3id3Get(FILE * inFile, Tag ** outTag)
{
  Tag * tag = new Tag();
  bool isTag = tag->get(inFile);
  if(outTag!=0) 
  	*outTag = tag ;
  else 
	delete tag;

  return isTag;
}

bool mp3id3Set(FILE * outFile, Tag * inTag, int verbose,
	       const char *title, const char *artist, const char *album, 
	       const char *year, const char *comment, int genre )
{
  bool mod=false;

  if(title!=0)
  {
  	inTag->setTitle(title);
	if(verbose) printf("title set to: %s\n", title);
	mod=true;
  }
  if(artist!=0)
  {
  	inTag->setArtist(artist);
	if(verbose) printf("artist set to: %s\n", artist);
	mod=true;
  }
  if(album!=0)
  {
  	inTag->setAlbum(album);
	if(verbose) printf("album set to: %s\n", album);
	mod=true;
  }
  if(year!=0)
  {
  	inTag->setYear(year);
	if(verbose) printf("year set to: %s\n", year);
	mod=true;
  }
  if(comment!=0)
  {
  	inTag->setComment(comment);
	if(verbose) printf("comment set to: %s\n", comment);
	mod=true;
  }
  if(genre!=0)
  {
  	inTag->setGenre(genre);
	if(verbose) printf("Genre set to: %d -> %s\n", genre, Tag::genres[genre]);
	mod=true;
  }
  if(mod)
  {
	if(verbose) printf("try to write new inTag\n");
	if(!inTag->set(outFile))
	{
		fprintf(stderr, "ERROR, while modifing inTag\n");
		mod = false;
	}
  }
  
  return mod;
}
